/*
 * Copyright (c) 2014-2016, Siemens AG. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef ALGORITHMS_CPP_TEST_SCAN_TEST_H_
#define ALGORITHMS_CPP_TEST_SCAN_TEST_H_

#include <partest/partest.h>

/**
 * Provides tests for the Scan method.
 */
class ScanTest : public partest::TestCase {
 public:
  /**
   * Creates test units.
   */
  ScanTest();

 private:
  /**
   * Tests the compatibility with different data structures.
   */
  void TestDataStructures();

  /**
   * Tests the scan function with the transform interface.
   */
  void TestTransform();

  /**
   * Tests the compatibility with function pointers.
   */
  void TestFunctionPointers();

  /**
   * Tests setting various ranges to be iterated.
   */
  void TestRanges();

  /**
   * Tests various block sizes for the workers.
   */
  void TestBlockSizes();

  /**
   * Tests setting policies (without checking their actual execution).
   */
  void TestPolicy();

  /**
   * Stress tests by giving work for all workers.
   */
  void StressTest();

  static const size_t kCountSize = 5;
};

#endif  // ALGORITHMS_CPP_TEST_SCAN_TEST_H_
